# Workshop Forests in HPC

![](general/img/gwdg_forest_logo.png)

[[_TOC_]]

# Analyzing lidar data on an HPC cluster

The course material for this part can be found in the vignette of the provided R package. An HTML version of that vignette can be found inside the R package: `forestHPC/vignettes/r-on-hpc.html`.

# Set Up for Deep Learning Part
We will create a `conda` environment containing all important packages and make it usable in the [jupyter HPC service](`https://jupyter-hpc.gwdg.de`) . 

💎 These sections are optional and are help in case you are stuck at a particular step.

## Creating a Conda Environment for Deep Learning (Once)
To follow along with the deep learning example, you need to to have the correct Python packages installed. On the cluster, these always live in a [conda environment](https://conda.io/projects/conda/en/latest/user-guide/tasks/manage-environments.html).

To set up the environment, first log into the frontend. Then create a [conda environment](https://info.gwdg.de/wiki/doku.php?id=wiki:hpc:conda) and install the required packages listed in `PointNet_Example/requirements.txt` with the following: 

First, clone the repo and create a conda environment:
```
git clone https://gitlab-ce.gwdg.de/hauke.gronenberg/workshop-forests-in-hpc.git
cd  workshop-forests-in-hpc/Pointnet_Example 
module load anaconda3
conda create -n test_pointnet python=3.9
```

Then activate the environment and install the requirements:
```
source activate test_pointnet
pip install -r requirements.txt
```

The test_pointnet `conda` environment now contains all packages that we need to run the script with.

💎 If you get something similar to `ERROR: No matching distribution found for ...`, one solution is to ease the requirements. You can do this by `nano Pointnet_Example/requirements.txt` and removing the version number for a package, so instead of `package==versionnumber` only `package` is displayed. If you do not require a specific version number, compatibility issues can be avoided, but it might also break things later on (if your code requires a specific version number).

💎 If you get an error during the training with the GPU such as `RuntimeError: CUDA error: CUBLAS_STATUS_NOT_SUPPORTED when calling cublasSgemmStridedBatched` later, you can solve it by _downgrading_ the PyTorch library with `pip install pytorch==1.12.1 torchvision==0.13.1`. 

## Python as a Kernel in Jupyter notebook

To use the `conda` environment as a kernel in the jupyter notebook, we need to register it once. 

#### Conda Environment as Kernel (Once)
You can run jupyter instances at `https://jupyter-hpc.gwdg.de`. What we need now is to make the environment available as a jupyter kernel [(see here for the documentation)](https://docs.gwdg.de/doku.php?id=en:services:application_services:jupyter:start).

Open a new jupyter instance, click top right on `New → Terminal`. 

Run the following in this newly created terminal:

```
. /opt/conda/etc/profile.d/conda.sh # note the dot at the beginning
conda activate test_pointnet
python3 -m ipykernel install --user --name test_pointnet --display-name "Python (test_pointnet)"
```

💎 You can check that this conda environment is now as a kernel available, it should be listed if you run `jupyter kernelspec list`.

💎 Make sure that the generated file is correct by with `cat ~/.local/share/jupyter/kernels/test_pointnet/kernel.json`. The `argv` argument should contain a path that both contains your username and python that looks similar to `"/usr/users/dmuelle3/.conda/envs/test_pointnet/bin/python"`. 

If this is *not* the case, use the following (replace the `"argv"` argument by your exact conda path):

```
echo "{
 \"argv\": [
  \"/usr/users/dmuelle3/.conda/envs/test_pointnet/bin/python\",
  \"-m\",
  \"ipykernel_launcher\",
  \"-f\",
  \"{connection_file}\"
 ],
 \"display_name\": \"Python (test_pointnet)\",
 \"language\": \"python\"
}" > ~/.local/share/jupyter/kernels/test_pointnet/kernel.json
```

Now restart the jupyter notebook service (top right `Control Panel -> Stop My Server`). You only need to register the environment as a kernel once, so now you can simply select it for a notebook as described below.

#### Opening Jupyter Notebook (Always)
Now you are ready to open the jupyter notebook with a specific kernel. Start a job at `https://jupyter-hpc.gwdg.de`. 
For this workshop, we will use the `IPython Parallel`. If you cloned this repository in your home directory, set the default home directory of the jupyter (Jupyter Notebook's Home directory) to `$HOME`.
Once you launch the notebook, go under the top bar `Kernel -> Change Kernel` to select your installed `conda` environment.

# Contact and Citation

If you have any questions or if you encounter any issues, please let us now:

- hauke.kirchner@gwdg.de
- dorothea.sommer@gwdg.de

```
@software{forests-in-hpc,
  author = {Kirchner, Hauke and Sommer, Dorothea},
  month = {12},
  year = {2022},
  title = {{Forests in HPC}},
  url = {https://gitlab-ce.gwdg.de/hpc-team-public/workshop-forests-in-hpc},
  version = {1.0}
}
```
