# this evaluation of the SLURM command sacct was deeply inspired by
# https://github.com/CSCfi/slurm-stats
#
# Creation of Statistics with SLURM
# sacct --format=JobID,JobIDRaw,JobName,User,Group,Partition,MaxRSS,MaxPages,AveCPU,MaxDiskWrite,MaxDiskRead,MaxVMSize,NTasks,AllocCPUS,Submit,Start,Elapsed,End,State,ExitCode,ReqMem,Timelimit -s BF,CA,CD,F,NF,PR,TO -P -a -j <job_id> > <job_id>_sacct.txt
analyse_sacct_out <- function(path){

  # Read the input file containing raw data
  dt = data.table::fread(path,header=T,sep="|")

  dt <- dt[which(dt$JobName == "batch"),]

  # Convert the elapsed times (run time and time limit) into seconds
  dt$Elapsed=as.numeric(unlist(lapply(dt$Elapsed,getelapsed)))

  # Convert the formatted byte values to standard bytes
  dt$MaxVMSize=convb(dt$MaxVMSize)
  dt$MaxRSS=convb(dt$MaxRSS)

  dt$JobID <- unlist(lapply(dt$JobID, rename_jobid)) %>%
    substr(start = 9, stop = 11) %>% as.numeric() %>% as.factor()

  # convert to GiB
  #dt$MaxRSS <- round(dt$MaxRSS / 1024^3,2)

  # convert to GiB
  #dt$MaxRSS <- round(dt$MaxRSS / 10^9,2)
  #dt$MaxVMSize <- round(dt$MaxVMSize / 10^9,2)

  #convert to minutes
  #dt$Elapsed <- round(dt$Elapsed / 60,2)

  return(dt)
}

# helper function
getelapsed <- function(x){
  str<-unlist(strsplit(as.character(x),'[-:]'))
  secs = 0
  if (length(str)==4) {
    secs <- as.numeric(str[1])*86400
    secs <- secs + as.numeric(str[2])*3600
    secs <- secs + as.numeric(str[3])*60
    secs <- secs + as.numeric(str[4])
  } else if (length(str)==3) {
    secs <- as.numeric(str[1])*3600
    secs <- secs + as.numeric(str[2])*60
    secs <- secs + as.numeric(str[3])
  }
  return(secs)
}

convb <- function(x){
  ptn <- "(\\d*(.\\d+)*)(.*)"
  num  <- as.numeric(sub(ptn, "\\1", x))
  unit <- sub(ptn, "\\3", x)
  unit[unit==""] <- "1"

  mult <- c("1"=1, "K"=1024, "M"=1024^2, "G"=1024^3)
  num * unname(mult[unit])
}

rename_jobid <- function(x){
  if(nchar(x) == 15){
    gsub('^(.{8})(.*)$', '\\100\\2', x)
  }else if(nchar(x) == 16){
    gsub('^(.{8})(.*)$', '\\10\\2', x)
  }else{
    x
  }
}
